import {
  defineComponent,
  nextTick,
  onMounted,
  reactive,
  ref,
  watch,
  toRefs
} from 'vue';
import styles from './index.module.less';
import {
  NButton,
  NForm,
  NFormItem,
  NImage,
  NPopselect,
  NSpace
} from 'naive-ui';
// import iconAdd from '../../images/icon-add.png';
import TheSearch from '/src/components/TheSearch';
import { resourceTypeArray } from '/src/utils/searchArray';
import { useCatchStore } from '/src/store/modules/catchData';
import { audioPlayType } from '/src/utils/contants';
// import isCollaose from '../../images/isCollaose.png';

const ChildNodeSearch = defineComponent({
  name: 'ChildNodeSearch',
  props: {
    activeRow: {
      type: Object,
      default: () => ({})
    },
    list: {
      type: Array,
      default: () => []
    }
  },
  emits: ['selectChildTag'],
  setup(props, { emit }) {
    const { activeRow } = toRefs(props);
    const selectItem = ref({});

    watch(
      () => props.activeRow,
      () => {
        activeRow.value = props.activeRow;
        selectItem.value = {};
      }
    );
    return () => (
      <>
        {activeRow.value?.id && (
          <>
            <NFormItem label={activeRow.value.columnName + '：'}>
              <NSpace class={styles.spaceSection}>
                {activeRow.value?.children.map((subject: any) => (
                  <span
                    class={[
                      styles.textBtn,
                      (activeRow.value.activeIndex || '') == subject.id &&
                        styles.textBtnActive
                    ]}
                    onClick={() => {
                      activeRow.value.activeIndex = subject.id;
                      let children: any;
                      let columnName = '';
                      if (subject.children) {
                        children = [
                          {
                            columnName: subject.children[0].columnName,
                            name: '全部',
                            id: ''
                          },
                          ...subject.children
                        ];
                        columnName = subject.children[0].columnName;

                        selectItem.value = {
                          ...subject,
                          columnName,
                          activeIndex: '',
                          children
                        };
                      } else {
                        selectItem.value = {};
                      }
                      emit('selectChildTag', activeRow.value.activeIndex);
                    }}>
                    {subject.name}
                  </span>
                ))}
              </NSpace>
            </NFormItem>

            <ChildNodeSearch
              activeRow={selectItem.value}
              onSelectChildTag={(item: any) => {
                // console.log(
                //   activeRow.value.activeIndex,
                //   'activeRow.value.activeIndex'
                // );
                emit('selectChildTag', item || activeRow.value.activeIndex);
              }}
            />
          </>
        )}
      </>
    );
  }
});

export default defineComponent({
  name: 'search-group',
  emits: ['search', 'add'],
  expose: ['init'],
  setup(props, { emit }) {
    const catchStore = useCatchStore();
    const forms = reactive({
      type: 'MUSIC', //
      name: '',
      // grade: null as any,
      audioPlayTypes: '',
      bookVersionId: null as any,
      // musicSheetCategoriesId: null as any,
      subjectId: null
    });
    const state = reactive({
      tempSubjectId: null
    });
    const data = reactive({
      audioPlayTypeList: [] as any, // 场景
      selectParents: {}, // 选中的数据
      tags: [] as any[],
      tagActiveId: '' as any,
      tagActive: {} as any,
      childSelectId: null as any
    });

    const onSearch = () => {
      emit('search', {
        ...forms,
        subjectId: forms.audioPlayTypes !== 'SING' ? forms.subjectId : null,
        audioPlayTypes: forms.audioPlayTypes
          ? forms.audioPlayTypes === 'PLAY_SING'
            ? ['PLAY', 'SING']
            : [forms.audioPlayTypes]
          : [],
        bookVersionId: data.childSelectId || data.tagActiveId
      });
    };
    const collapseWrapRef = ref();
    const divDomList = ref([] as any);
    const orginHeight = ref(0);
    const line = ref(0);
    const isCollapse = ref(false);
    const loadingCollapse = ref(false); // 是否加载完成
    // const musicCateRef = (el: any) => {
    //   if (el?.selfElRef) {
    //     divDomList.value.push(el.selfElRef.parentNode);
    //   }
    // };
    // const setCollapse = (flag: boolean) => {
    //   isCollapse.value = flag;
    //   getLive();
    // };
    const getLive = () => {
      try {
        divDomList.value = [...new Set(divDomList.value)];
        let offsetLeft = -1;
        divDomList.value.forEach((item: any, index: number) => {
          if (index === 0) {
            line.value = 1;
            offsetLeft = item.offsetLeft;
          } else if (item.offsetLeft === offsetLeft && index != 0) {
            // 如果某个标签的offsetLeft和第一个标签的offsetLeft相等  说明增加了一行
            line.value++;
          }
          if (!isCollapse.value) {
            if (line.value > 1) {
              //从第3行开始 隐藏标签
              item.style.display = 'none';
              // 显示展开按钮  class名chu是在前面动态添加的
            } else {
              item.style.display = 'block';
            }
          } else {
            item.style.display = 'block';
          }
        });
        loadingCollapse.value = true;
      } catch {
        //
      }
    };

    const selectChildObj = (item: any) => {
      const obj: any = {};
      item?.forEach((child: any) => {
        if (child.id === forms.subjectId) {
          obj.selected = true;
          obj.name = child.name;
        }
      });
      return obj;
    };

    const _initTags = () => {
      const tags = catchStore.getMusicTagTree;
      data.tags = [
        {
          columnName: tags[0].columnName,
          name: '全部',
          id: ''
        },
        ...tags
      ];
      data.tagActiveId = data.tags[0].id;
    };

    const changeTag = (item: any) => {
      data.tagActiveId = item.id;
      data.childSelectId = null;
      let children: any;
      let columnName = '';
      if (item.children) {
        children = [
          {
            columnName: item.children[0].columnName,
            name: '全部',
            id: ''
          },
          ...item.children
        ];
        columnName = item.children[0].columnName;
        data.selectParents = {
          ...item,
          columnName,
          activeIndex: '',
          children
        };
      } else {
        data.selectParents = {};
      }

      onSearch();
    };

    /** 默认选中第一个声部 */
    const formatFirstSubject = () => {
      const tempSubjects = catchStore.getSubjectInstrumentOnly;
      if (tempSubjects.length > 0) {
        const firstSubject = tempSubjects[0];
        if (firstSubject.instruments && firstSubject.instruments.length > 1) {
          state.tempSubjectId = firstSubject.instruments[0]?.value;
          forms.subjectId = firstSubject.instruments[0]?.value;
        } else {
          forms.subjectId = firstSubject.value;
        }
      }
    };
    onMounted(async () => {
      // 场景
      const tempAudio = Object.keys(audioPlayType).map(key => {
        return {
          value: key,
          name: audioPlayType[key]
        };
      });
      data.audioPlayTypeList = [{ name: '全部', value: '' }, ...tempAudio];

      // console.log('加载');
      // 获取教材分类列表
      // await catchStore.getMusicSheetCategory();
      await catchStore.getMusicTagTreeApi();
      _initTags();
      // console.log(data, 'data');
      // 获取声部列表
      await catchStore.getSubjects();

      formatFirstSubject();
      if (forms.type === 'MUSIC') {
        orginHeight.value = collapseWrapRef.value?.offsetHeight;
        // hiddenHeight.value = collapseWrapRef.value.offsetHeight / line.value;
        // 默认隐藏
        getLive();
      }

      onSearch();
    });
    return () => (
      <div class={styles.searchGroup}>
        <div class={[styles.searchCatatory]}>
          <NSpace size="small" class={styles.btnType}>
            {resourceTypeArray.map((item: any) => (
              <NButton
                type={forms.type === item.value ? 'primary' : 'default'}
                secondary={forms.type === item.value ? false : true}
                round
                size="small"
                focusable={false}
                onClick={() => {
                  forms.type = item.value;
                  forms.audioPlayTypes = '';
                  data.tagActiveId = '';
                  data.childSelectId = null;
                  data.selectParents = {};

                  formatFirstSubject();

                  onSearch();

                  nextTick(() => {
                    if (forms.type === 'MUSIC') {
                      orginHeight.value = collapseWrapRef.value?.offsetHeight;
                      // hiddenHeight.value =
                      //   collapseWrapRef.value.offsetHeight / line.value || 0;
                      // 默认隐藏
                      getLive();
                    } else {
                      divDomList.value = [];
                    }
                  });
                }}>
                {item.label}
              </NButton>
            ))}
          </NSpace>

          {/* <NButton
            type="primary"
            class={styles.addTrain}
            focusable={false}
            strong
            onClick={() => emit('add')}>
            <img src={iconAdd} />
            添加自定义教材
          </NButton> */}
        </div>
        <NForm labelAlign="left" labelPlacement="left">
          {forms.type === 'MUSIC' && (
            <>
              {data.tags.length > 0 && (
                <NFormItem label={data.tags[0]?.columnName + '：'}>
                  <NSpace class={styles.spaceSection}>
                    {data.tags.map((subject: any) => (
                      <span
                        class={[
                          styles.textBtn,
                          data.tagActiveId === subject.id &&
                            styles.textBtnActive
                        ]}
                        onClick={() => {
                          changeTag(subject);
                        }}>
                        {subject.name}
                      </span>
                    ))}
                  </NSpace>
                </NFormItem>
              )}

              <ChildNodeSearch
                activeRow={data.selectParents}
                onSelectChildTag={(val: any) => {
                  data.childSelectId = val;
                  onSearch();
                }}
              />
            </>
          )}

          <NFormItem label="场景：">
            <NSpace class={styles.spaceSection}>
              {data.audioPlayTypeList.map((subject: any) => (
                <span
                  class={[
                    styles.textBtn,

                    forms.audioPlayTypes === subject.value &&
                      styles.textBtnActive
                  ]}
                  onClick={() => {
                    forms.audioPlayTypes = subject.value;
                    // if (subject.value === 'SING') {
                    //   state.tempSubjectId = null;
                    //   forms.subjectId = null;
                    // } else {
                    //   formatFirstSubject();
                    // }
                    onSearch();
                  }}>
                  {subject.name}
                </span>
              ))}
            </NSpace>
          </NFormItem>

          {forms.audioPlayTypes !== 'SING' && (
            <NFormItem label="乐器：">
              <NSpace class={styles.spaceSection}>
                {catchStore.getSubjectInstrumentOnly.map((subject: any) =>
                  subject.instruments && subject.instruments.length > 1 ? (
                    <NPopselect
                      options={subject.instruments}
                      trigger="hover"
                      scrollable
                      v-model:value={state.tempSubjectId}
                      onUpdate:value={() => {
                        forms.subjectId = state.tempSubjectId;
                        onSearch();
                      }}
                      key={subject.value}
                      class={[styles.popSelect]}>
                      <span
                        class={[
                          styles.textBtn,
                          selectChildObj(subject.instruments).selected &&
                            styles.textBtnActive
                        ]}>
                        {selectChildObj(subject.instruments).name ||
                          subject.name}
                        <i class={styles.iconArrow}></i>
                      </span>
                    </NPopselect>
                  ) : (
                    <span
                      class={[
                        styles.textBtn,

                        forms.subjectId === subject.value &&
                          styles.textBtnActive
                      ]}
                      onClick={() => {
                        forms.subjectId = subject.value;

                        state.tempSubjectId = null;
                        onSearch();
                      }}>
                      {subject.name}
                    </span>
                  )
                )}
              </NSpace>
            </NFormItem>
          )}

          <TheSearch
            class={styles.inputSearch}
            round
            onSearch={(val: string) => {
              forms.name = val;
              onSearch();
            }}
          />
        </NForm>
      </div>
    );
  }
});
